"""Filter types from schema/types/filters.graphql."""

from __future__ import annotations

from typing import Any

from .base import StashInput, StashResult
from .enums import (
    CircumisedEnum,
    CriterionModifier,
    FilterMode,
    GenderEnum,
    OrientationEnum,
    ResolutionEnum,
    SortDirectionEnum,
)
from .unset import UNSET, UnsetType


class FindFilterType(StashInput):
    """Input for find filter."""

    q: str | None | UnsetType = UNSET  # String
    page: int | None | UnsetType = UNSET  # Int
    per_page: int | None | UnsetType = UNSET  # Int (-1 for all, default 25)
    sort: str | None | UnsetType = UNSET  # String
    direction: SortDirectionEnum | None | UnsetType = UNSET  # SortDirectionEnum


class SavedFindFilterType(StashInput):
    """Saved find filter type."""

    q: str | None | UnsetType = UNSET  # String
    page: int | None | UnsetType = UNSET  # Int
    per_page: int | None | UnsetType = UNSET  # Int (-1 for all, default 25)
    sort: str | None | UnsetType = UNSET  # String
    direction: SortDirectionEnum | None | UnsetType = UNSET  # SortDirectionEnum


class ResolutionCriterionInput(StashInput):
    """Input for resolution criterion."""

    value: ResolutionEnum  # ResolutionEnum!
    modifier: CriterionModifier  # CriterionModifier!


class OrientationCriterionInput(StashInput):
    """Input for orientation criterion."""

    value: list[OrientationEnum]  # [OrientationEnum!]!


class PHashDuplicationCriterionInput(StashInput):
    """Input for phash duplication criterion."""

    duplicated: bool | None | UnsetType = UNSET  # Boolean
    distance: int | None | UnsetType = UNSET  # Int


class StashIDCriterionInput(StashInput):
    """Input for StashID criterion."""

    endpoint: str | None | UnsetType = UNSET  # String
    stash_id: str | None | UnsetType = UNSET  # String
    modifier: CriterionModifier  # CriterionModifier!


class CustomFieldCriterionInput(StashInput):
    """Input for custom field criterion."""

    field: str  # String!
    value: list[Any] | None | UnsetType = UNSET  # [Any!]
    modifier: CriterionModifier  # CriterionModifier!


class StringCriterionInput(StashInput):
    """Input for string criterion."""

    value: str  # String!
    modifier: CriterionModifier  # CriterionModifier!


class IntCriterionInput(StashInput):
    """Input for integer criterion."""

    value: int  # Int!
    value2: int | None | UnsetType = UNSET  # Int
    modifier: CriterionModifier  # CriterionModifier!


class FloatCriterionInput(StashInput):
    """Input for float criterion."""

    value: float  # Float!
    value2: float | None | UnsetType = UNSET  # Float
    modifier: CriterionModifier  # CriterionModifier!


class MultiCriterionInput(StashInput):
    """Input for multi criterion."""

    value: list[str] | None | UnsetType = UNSET  # [ID!]
    modifier: CriterionModifier  # CriterionModifier!
    excludes: list[str] | None | UnsetType = UNSET  # [ID!]


class GenderCriterionInput(StashInput):
    """Input for gender criterion."""

    value: GenderEnum | None | UnsetType = UNSET  # GenderEnum
    value_list: list[GenderEnum] | None | UnsetType = UNSET  # [GenderEnum!]
    modifier: CriterionModifier  # CriterionModifier!


class CircumcisionCriterionInput(StashInput):
    """Input for circumcision criterion."""

    value: list[CircumisedEnum]  # [CircumisedEnum!]!
    modifier: CriterionModifier  # CriterionModifier!


class HierarchicalMultiCriterionInput(StashInput):
    """Input for hierarchical multi criterion."""

    value: list[str]  # [ID!]!
    modifier: CriterionModifier  # CriterionModifier!
    depth: int | None | UnsetType = UNSET  # Int
    excludes: list[str] | None | UnsetType = UNSET  # [ID!]


class DateCriterionInput(StashInput):
    """Input for date criterion."""

    value: str  # String!
    value2: str | None | UnsetType = UNSET  # String
    modifier: CriterionModifier  # CriterionModifier!


class TimestampCriterionInput(StashInput):
    """Input for timestamp criterion."""

    value: str  # String!
    value2: str | None | UnsetType = UNSET  # String
    modifier: CriterionModifier  # CriterionModifier!


class PhashDistanceCriterionInput(StashInput):
    """Input for phash distance criterion."""

    value: str  # String!
    modifier: CriterionModifier  # CriterionModifier!
    distance: int | None | UnsetType = UNSET  # Int


class SavedFilter(StashResult):
    """Saved filter type."""

    id: str  # ID!
    mode: FilterMode | UnsetType = UNSET  # FilterMode!
    name: str | UnsetType = UNSET  # String!
    find_filter: SavedFindFilterType | None | UnsetType = UNSET  # SavedFindFilterType
    object_filter: dict[str, Any] | None | UnsetType = UNSET  # Map
    ui_options: dict[str, Any] | None | UnsetType = UNSET  # Map


class SaveFilterInput(StashInput):
    """Input for saving filter."""

    id: str | None | UnsetType = UNSET  # ID
    mode: FilterMode  # FilterMode!
    name: str  # String!
    find_filter: FindFilterType | None | UnsetType = UNSET  # FindFilterType
    object_filter: dict[str, Any] | None | UnsetType = UNSET  # Map
    ui_options: dict[str, Any] | None | UnsetType = UNSET  # Map


class DestroyFilterInput(StashInput):
    """Input for destroying filter."""

    id: str  # ID!


class SetDefaultFilterInput(StashInput):
    """Input for setting default filter."""

    mode: FilterMode  # FilterMode!
    find_filter: FindFilterType | None | UnsetType = UNSET  # FindFilterType
    object_filter: dict[str, Any] | None | UnsetType = UNSET  # Map
    ui_options: dict[str, Any] | None | UnsetType = UNSET  # Map


# Core filter types


class PerformerFilterType(StashInput):
    """Input for performer filter."""

    AND: PerformerFilterType | None | UnsetType = UNSET
    OR: PerformerFilterType | None | UnsetType = UNSET
    NOT: PerformerFilterType | None | UnsetType = UNSET
    name: StringCriterionInput | None | UnsetType = UNSET
    disambiguation: StringCriterionInput | None | UnsetType = UNSET
    details: StringCriterionInput | None | UnsetType = UNSET
    filter_favorites: bool | None | UnsetType = UNSET
    birth_year: IntCriterionInput | None | UnsetType = UNSET
    age: IntCriterionInput | None | UnsetType = UNSET
    ethnicity: StringCriterionInput | None | UnsetType = UNSET
    country: StringCriterionInput | None | UnsetType = UNSET
    eye_color: StringCriterionInput | None | UnsetType = UNSET
    height_cm: IntCriterionInput | None | UnsetType = UNSET
    measurements: StringCriterionInput | None | UnsetType = UNSET
    fake_tits: StringCriterionInput | None | UnsetType = UNSET
    penis_length: FloatCriterionInput | None | UnsetType = UNSET
    circumcised: CircumcisionCriterionInput | None | UnsetType = UNSET
    career_length: StringCriterionInput | None | UnsetType = UNSET
    tattoos: StringCriterionInput | None | UnsetType = UNSET
    piercings: StringCriterionInput | None | UnsetType = UNSET
    aliases: StringCriterionInput | None | UnsetType = UNSET
    gender: GenderCriterionInput | None | UnsetType = UNSET
    is_missing: str | None | UnsetType = UNSET
    tags: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    tag_count: IntCriterionInput | None | UnsetType = UNSET
    scene_count: IntCriterionInput | None | UnsetType = UNSET
    image_count: IntCriterionInput | None | UnsetType = UNSET
    gallery_count: IntCriterionInput | None | UnsetType = UNSET
    play_count: IntCriterionInput | None | UnsetType = UNSET
    o_counter: IntCriterionInput | None | UnsetType = UNSET
    stash_id_endpoint: StashIDCriterionInput | None | UnsetType = UNSET
    rating100: IntCriterionInput | None | UnsetType = UNSET
    url: StringCriterionInput | None | UnsetType = UNSET
    hair_color: StringCriterionInput | None | UnsetType = UNSET
    weight: IntCriterionInput | None | UnsetType = UNSET
    death_year: IntCriterionInput | None | UnsetType = UNSET
    studios: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    groups: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    performers: MultiCriterionInput | None | UnsetType = UNSET
    ignore_auto_tag: bool | None | UnsetType = UNSET
    birthdate: DateCriterionInput | None | UnsetType = UNSET
    death_date: DateCriterionInput | None | UnsetType = UNSET
    scenes_filter: SceneFilterType | None | UnsetType = UNSET
    images_filter: ImageFilterType | None | UnsetType = UNSET
    galleries_filter: GalleryFilterType | None | UnsetType = UNSET
    tags_filter: TagFilterType | None | UnsetType = UNSET
    created_at: TimestampCriterionInput | None | UnsetType = UNSET
    updated_at: TimestampCriterionInput | None | UnsetType = UNSET
    custom_fields: list[CustomFieldCriterionInput] | None | UnsetType = UNSET


class SceneMarkerFilterType(StashInput):
    """Input for scene marker filter."""

    tags: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    scene_tags: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    performers: MultiCriterionInput | None | UnsetType = UNSET
    scenes: MultiCriterionInput | None | UnsetType = UNSET
    duration: FloatCriterionInput | None | UnsetType = UNSET
    created_at: TimestampCriterionInput | None | UnsetType = UNSET
    updated_at: TimestampCriterionInput | None | UnsetType = UNSET
    scene_date: DateCriterionInput | None | UnsetType = UNSET
    scene_created_at: TimestampCriterionInput | None | UnsetType = UNSET
    scene_updated_at: TimestampCriterionInput | None | UnsetType = UNSET
    scene_filter: SceneFilterType | None | UnsetType = UNSET


class SceneFilterType(StashInput):
    """Input for scene filter."""

    AND: SceneFilterType | None | UnsetType = UNSET
    OR: SceneFilterType | None | UnsetType = UNSET
    NOT: SceneFilterType | None | UnsetType = UNSET
    id: IntCriterionInput | None | UnsetType = UNSET
    title: StringCriterionInput | None | UnsetType = UNSET
    code: StringCriterionInput | None | UnsetType = UNSET
    details: StringCriterionInput | None | UnsetType = UNSET
    director: StringCriterionInput | None | UnsetType = UNSET
    oshash: StringCriterionInput | None | UnsetType = UNSET
    checksum: StringCriterionInput | None | UnsetType = UNSET
    phash_distance: PhashDistanceCriterionInput | None | UnsetType = UNSET
    path: StringCriterionInput | None | UnsetType = UNSET
    file_count: IntCriterionInput | None | UnsetType = UNSET
    rating100: IntCriterionInput | None | UnsetType = UNSET
    organized: bool | None | UnsetType = UNSET
    o_counter: IntCriterionInput | None | UnsetType = UNSET
    duplicated: PHashDuplicationCriterionInput | None | UnsetType = UNSET
    resolution: ResolutionCriterionInput | None | UnsetType = UNSET
    orientation: OrientationCriterionInput | None | UnsetType = UNSET
    framerate: IntCriterionInput | None | UnsetType = UNSET
    bitrate: IntCriterionInput | None | UnsetType = UNSET
    video_codec: StringCriterionInput | None | UnsetType = UNSET
    audio_codec: StringCriterionInput | None | UnsetType = UNSET
    duration: IntCriterionInput | None | UnsetType = UNSET
    has_markers: str | None | UnsetType = UNSET
    is_missing: str | None | UnsetType = UNSET
    studios: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    groups: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    galleries: MultiCriterionInput | None | UnsetType = UNSET
    tags: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    tag_count: IntCriterionInput | None | UnsetType = UNSET
    performer_tags: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    performer_favorite: bool | None | UnsetType = UNSET
    performer_age: IntCriterionInput | None | UnsetType = UNSET
    performers: MultiCriterionInput | None | UnsetType = UNSET
    performer_count: IntCriterionInput | None | UnsetType = UNSET
    stash_id_endpoint: StashIDCriterionInput | None | UnsetType = UNSET
    url: StringCriterionInput | None | UnsetType = UNSET
    interactive: bool | None | UnsetType = UNSET
    interactive_speed: IntCriterionInput | None | UnsetType = UNSET
    captions: StringCriterionInput | None | UnsetType = UNSET
    resume_time: IntCriterionInput | None | UnsetType = UNSET
    play_count: IntCriterionInput | None | UnsetType = UNSET
    play_duration: IntCriterionInput | None | UnsetType = UNSET
    last_played_at: TimestampCriterionInput | None | UnsetType = UNSET
    date: DateCriterionInput | None | UnsetType = UNSET
    created_at: TimestampCriterionInput | None | UnsetType = UNSET
    updated_at: TimestampCriterionInput | None | UnsetType = UNSET
    galleries_filter: GalleryFilterType | None | UnsetType = UNSET
    performers_filter: PerformerFilterType | None | UnsetType = UNSET
    studios_filter: StudioFilterType | None | UnsetType = UNSET
    tags_filter: TagFilterType | None | UnsetType = UNSET
    groups_filter: GroupFilterType | None | UnsetType = UNSET
    markers_filter: SceneMarkerFilterType | None | UnsetType = UNSET
    files_filter: FileFilterType | None | UnsetType = UNSET


class GroupFilterType(StashInput):
    """Input for group filter."""

    AND: GroupFilterType | None | UnsetType = UNSET
    OR: GroupFilterType | None | UnsetType = UNSET
    NOT: GroupFilterType | None | UnsetType = UNSET
    name: StringCriterionInput | None | UnsetType = UNSET
    director: StringCriterionInput | None | UnsetType = UNSET
    synopsis: StringCriterionInput | None | UnsetType = UNSET
    duration: IntCriterionInput | None | UnsetType = UNSET
    rating100: IntCriterionInput | None | UnsetType = UNSET
    studios: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    is_missing: str | None | UnsetType = UNSET
    url: StringCriterionInput | None | UnsetType = UNSET
    performers: MultiCriterionInput | None | UnsetType = UNSET
    tags: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    tag_count: IntCriterionInput | None | UnsetType = UNSET
    date: DateCriterionInput | None | UnsetType = UNSET
    created_at: TimestampCriterionInput | None | UnsetType = UNSET
    updated_at: TimestampCriterionInput | None | UnsetType = UNSET
    o_counter: IntCriterionInput | None | UnsetType = UNSET
    containing_groups: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    sub_groups: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    containing_group_count: IntCriterionInput | None | UnsetType = UNSET
    sub_group_count: IntCriterionInput | None | UnsetType = UNSET
    scenes_filter: SceneFilterType | None | UnsetType = UNSET
    studios_filter: StudioFilterType | None | UnsetType = UNSET


class StudioFilterType(StashInput):
    """Input for studio filter."""

    AND: StudioFilterType | None | UnsetType = UNSET
    OR: StudioFilterType | None | UnsetType = UNSET
    NOT: StudioFilterType | None | UnsetType = UNSET
    name: StringCriterionInput | None | UnsetType = UNSET
    details: StringCriterionInput | None | UnsetType = UNSET
    parents: MultiCriterionInput | None | UnsetType = UNSET
    stash_id_endpoint: StashIDCriterionInput | None | UnsetType = UNSET
    tags: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    is_missing: str | None | UnsetType = UNSET
    rating100: IntCriterionInput | None | UnsetType = UNSET
    favorite: bool | None | UnsetType = UNSET
    scene_count: IntCriterionInput | None | UnsetType = UNSET
    image_count: IntCriterionInput | None | UnsetType = UNSET
    gallery_count: IntCriterionInput | None | UnsetType = UNSET
    tag_count: IntCriterionInput | None | UnsetType = UNSET
    url: StringCriterionInput | None | UnsetType = UNSET
    aliases: StringCriterionInput | None | UnsetType = UNSET
    child_count: IntCriterionInput | None | UnsetType = UNSET
    ignore_auto_tag: bool | None | UnsetType = UNSET
    scenes_filter: SceneFilterType | None | UnsetType = UNSET
    images_filter: ImageFilterType | None | UnsetType = UNSET
    galleries_filter: GalleryFilterType | None | UnsetType = UNSET
    created_at: TimestampCriterionInput | None | UnsetType = UNSET
    updated_at: TimestampCriterionInput | None | UnsetType = UNSET


class GalleryFilterType(StashInput):
    """Input for gallery filter."""

    AND: GalleryFilterType | None | UnsetType = UNSET
    OR: GalleryFilterType | None | UnsetType = UNSET
    NOT: GalleryFilterType | None | UnsetType = UNSET
    id: IntCriterionInput | None | UnsetType = UNSET
    title: StringCriterionInput | None | UnsetType = UNSET
    details: StringCriterionInput | None | UnsetType = UNSET
    checksum: StringCriterionInput | None | UnsetType = UNSET
    path: StringCriterionInput | None | UnsetType = UNSET
    file_count: IntCriterionInput | None | UnsetType = UNSET
    is_missing: str | None | UnsetType = UNSET
    is_zip: bool | None | UnsetType = UNSET
    rating100: IntCriterionInput | None | UnsetType = UNSET
    organized: bool | None | UnsetType = UNSET
    average_resolution: ResolutionCriterionInput | None | UnsetType = UNSET
    has_chapters: str | None | UnsetType = UNSET
    scenes: MultiCriterionInput | None | UnsetType = UNSET
    studios: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    tags: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    tag_count: IntCriterionInput | None | UnsetType = UNSET
    performer_tags: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    performers: MultiCriterionInput | None | UnsetType = UNSET
    performer_count: IntCriterionInput | None | UnsetType = UNSET
    performer_favorite: bool | None | UnsetType = UNSET
    performer_age: IntCriterionInput | None | UnsetType = UNSET
    image_count: IntCriterionInput | None | UnsetType = UNSET
    url: StringCriterionInput | None | UnsetType = UNSET
    date: DateCriterionInput | None | UnsetType = UNSET
    created_at: TimestampCriterionInput | None | UnsetType = UNSET
    updated_at: TimestampCriterionInput | None | UnsetType = UNSET
    code: StringCriterionInput | None | UnsetType = UNSET
    photographer: StringCriterionInput | None | UnsetType = UNSET
    scenes_filter: SceneFilterType | None | UnsetType = UNSET
    images_filter: ImageFilterType | None | UnsetType = UNSET
    performers_filter: PerformerFilterType | None | UnsetType = UNSET
    studios_filter: StudioFilterType | None | UnsetType = UNSET
    tags_filter: TagFilterType | None | UnsetType = UNSET
    files_filter: FileFilterType | None | UnsetType = UNSET
    folders_filter: FolderFilterType | None | UnsetType = UNSET


class TagFilterType(StashInput):
    """Input for tag filter."""

    AND: TagFilterType | None | UnsetType = UNSET
    OR: TagFilterType | None | UnsetType = UNSET
    NOT: TagFilterType | None | UnsetType = UNSET
    name: StringCriterionInput | None | UnsetType = UNSET
    sort_name: StringCriterionInput | None | UnsetType = UNSET
    aliases: StringCriterionInput | None | UnsetType = UNSET
    favorite: bool | None | UnsetType = UNSET
    description: StringCriterionInput | None | UnsetType = UNSET
    is_missing: str | None | UnsetType = UNSET
    scene_count: IntCriterionInput | None | UnsetType = UNSET
    image_count: IntCriterionInput | None | UnsetType = UNSET
    gallery_count: IntCriterionInput | None | UnsetType = UNSET
    performer_count: IntCriterionInput | None | UnsetType = UNSET
    studio_count: IntCriterionInput | None | UnsetType = UNSET
    movie_count: IntCriterionInput | None | UnsetType = UNSET
    group_count: IntCriterionInput | None | UnsetType = UNSET
    marker_count: IntCriterionInput | None | UnsetType = UNSET
    parents: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    children: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    parent_count: IntCriterionInput | None | UnsetType = UNSET
    child_count: IntCriterionInput | None | UnsetType = UNSET
    ignore_auto_tag: bool | None | UnsetType = UNSET
    scenes_filter: SceneFilterType | None | UnsetType = UNSET
    images_filter: ImageFilterType | None | UnsetType = UNSET
    galleries_filter: GalleryFilterType | None | UnsetType = UNSET
    created_at: TimestampCriterionInput | None | UnsetType = UNSET
    updated_at: TimestampCriterionInput | None | UnsetType = UNSET


class ImageFilterType(StashInput):
    """Input for image filter."""

    AND: ImageFilterType | None | UnsetType = UNSET
    OR: ImageFilterType | None | UnsetType = UNSET
    NOT: ImageFilterType | None | UnsetType = UNSET
    title: StringCriterionInput | None | UnsetType = UNSET
    details: StringCriterionInput | None | UnsetType = UNSET
    id: IntCriterionInput | None | UnsetType = UNSET
    checksum: StringCriterionInput | None | UnsetType = UNSET
    path: StringCriterionInput | None | UnsetType = UNSET
    file_count: IntCriterionInput | None | UnsetType = UNSET
    rating100: IntCriterionInput | None | UnsetType = UNSET
    date: DateCriterionInput | None | UnsetType = UNSET
    url: StringCriterionInput | None | UnsetType = UNSET
    organized: bool | None | UnsetType = UNSET
    o_counter: IntCriterionInput | None | UnsetType = UNSET
    resolution: ResolutionCriterionInput | None | UnsetType = UNSET
    orientation: OrientationCriterionInput | None | UnsetType = UNSET
    is_missing: str | None | UnsetType = UNSET
    studios: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    tags: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    tag_count: IntCriterionInput | None | UnsetType = UNSET
    performer_tags: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    performers: MultiCriterionInput | None | UnsetType = UNSET
    performer_count: IntCriterionInput | None | UnsetType = UNSET
    performer_favorite: bool | None | UnsetType = UNSET
    performer_age: IntCriterionInput | None | UnsetType = UNSET
    galleries: MultiCriterionInput | None | UnsetType = UNSET
    created_at: TimestampCriterionInput | None | UnsetType = UNSET
    updated_at: TimestampCriterionInput | None | UnsetType = UNSET
    code: StringCriterionInput | None | UnsetType = UNSET
    photographer: StringCriterionInput | None | UnsetType = UNSET
    galleries_filter: GalleryFilterType | None | UnsetType = UNSET
    performers_filter: PerformerFilterType | None | UnsetType = UNSET
    studios_filter: StudioFilterType | None | UnsetType = UNSET
    tags_filter: TagFilterType | None | UnsetType = UNSET
    files_filter: FileFilterType | None | UnsetType = UNSET


class FileFilterType(StashInput):
    """Input for file filter."""

    AND: FileFilterType | None | UnsetType = UNSET
    OR: FileFilterType | None | UnsetType = UNSET
    NOT: FileFilterType | None | UnsetType = UNSET
    path: StringCriterionInput | None | UnsetType = UNSET
    basename: StringCriterionInput | None | UnsetType = UNSET
    dir: StringCriterionInput | None | UnsetType = UNSET
    parent_folder: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    zip_file: MultiCriterionInput | None | UnsetType = UNSET
    mod_time: TimestampCriterionInput | None | UnsetType = UNSET
    duplicated: PHashDuplicationCriterionInput | None | UnsetType = UNSET
    hashes: list[FingerprintFilterInput] | None | UnsetType = UNSET
    video_file_filter: VideoFileFilterInput | None | UnsetType = UNSET
    image_file_filter: ImageFileFilterInput | None | UnsetType = UNSET
    scene_count: IntCriterionInput | None | UnsetType = UNSET
    image_count: IntCriterionInput | None | UnsetType = UNSET
    gallery_count: IntCriterionInput | None | UnsetType = UNSET
    scenes_filter: SceneFilterType | None | UnsetType = UNSET
    images_filter: ImageFilterType | None | UnsetType = UNSET
    galleries_filter: GalleryFilterType | None | UnsetType = UNSET
    created_at: TimestampCriterionInput | None | UnsetType = UNSET
    updated_at: TimestampCriterionInput | None | UnsetType = UNSET


class FolderFilterType(StashInput):
    """Input for folder filter."""

    AND: FolderFilterType | None | UnsetType = UNSET
    OR: FolderFilterType | None | UnsetType = UNSET
    NOT: FolderFilterType | None | UnsetType = UNSET
    path: StringCriterionInput | None | UnsetType = UNSET
    parent_folder: HierarchicalMultiCriterionInput | None | UnsetType = UNSET
    zip_file: MultiCriterionInput | None | UnsetType = UNSET
    mod_time: TimestampCriterionInput | None | UnsetType = UNSET
    gallery_count: IntCriterionInput | None | UnsetType = UNSET
    files_filter: FileFilterType | None | UnsetType = UNSET
    galleries_filter: GalleryFilterType | None | UnsetType = UNSET
    created_at: TimestampCriterionInput | None | UnsetType = UNSET
    updated_at: TimestampCriterionInput | None | UnsetType = UNSET


class VideoFileFilterInput(StashInput):
    """Input for video file filter."""

    resolution: ResolutionCriterionInput | None | UnsetType = UNSET
    orientation: OrientationCriterionInput | None | UnsetType = UNSET
    framerate: IntCriterionInput | None | UnsetType = UNSET
    bitrate: IntCriterionInput | None | UnsetType = UNSET
    format: StringCriterionInput | None | UnsetType = UNSET
    video_codec: StringCriterionInput | None | UnsetType = UNSET
    audio_codec: StringCriterionInput | None | UnsetType = UNSET
    duration: IntCriterionInput | None | UnsetType = UNSET
    captions: StringCriterionInput | None | UnsetType = UNSET
    interactive: bool | None | UnsetType = UNSET
    interactive_speed: IntCriterionInput | None | UnsetType = UNSET


class ImageFileFilterInput(StashInput):
    """Input for image file filter."""

    format: StringCriterionInput | None | UnsetType = UNSET
    resolution: ResolutionCriterionInput | None | UnsetType = UNSET
    orientation: OrientationCriterionInput | None | UnsetType = UNSET


class FingerprintFilterInput(StashInput):
    """Input for fingerprint filter."""

    type: str  # String!
    value: str  # String!
    distance: int | None | UnsetType = UNSET  # Int (defaults to 0)



"""File types from schema/types/file.graphql."""

from __future__ import annotations

from datetime import datetime
from typing import Any

from pydantic import BaseModel, Field, field_validator

from .base import StashInput, StashObject, StashResult
from .scalars import Time
from .unset import UNSET, UnsetType


def fingerprint_resolver(parent: BaseFile, type: str) -> str:
    """Resolver for fingerprint field.

    Args:
        parent: The BaseFile instance (automatically passed by strawberry)
        type: The fingerprint type to look for

    Returns:
        The fingerprint value for the given type, or empty string if not found.
        This matches the GraphQL schema which defines the return type as String! (non-nullable).
    """
    # Handle UNSET case
    if isinstance(parent.fingerprints, UnsetType):
        return ""

    for fp in parent.fingerprints:
        if fp.type_ == type:
            return fp.value
    return ""  # Return empty string instead of None to match GraphQL schema


class SetFingerprintsInput(StashInput):
    """Input for setting fingerprints."""

    type_: str = Field(alias="type")  # String! - aliased to avoid built-in conflict
    value: str | None | UnsetType = UNSET  # String


class FileSetFingerprintsInput(StashInput):
    """Input for setting file fingerprints."""

    id: str  # ID!
    fingerprints: list[SetFingerprintsInput]  # [SetFingerprintsInput!]!


class MoveFilesInput(StashInput):
    """Input for moving files."""

    ids: list[str]  # [ID!]!
    destination_folder: str | None | UnsetType = UNSET  # String
    destination_folder_id: str | None | UnsetType = UNSET  # ID
    destination_basename: str | None | UnsetType = UNSET  # String


class Fingerprint(BaseModel):
    """Fingerprint type from schema/types/file.graphql."""

    type_: str = Field(alias="type")  # String! - aliased to avoid built-in conflict
    value: str  # String!


class BaseFile(StashObject):
    """Base interface for all file types from schema/types/file.graphql.

    Note: Inherits from StashObject since it has id, created_at, and updated_at
    fields in the schema, matching the common pattern."""

    __type_name__ = "BaseFile"

    # Required fields
    path: str | UnsetType = UNSET  # String!
    basename: str | UnsetType = UNSET  # String!
    parent_folder: Folder | UnsetType = UNSET  # Folder!
    mod_time: datetime | UnsetType = UNSET  # Time!
    size: int | UnsetType = UNSET  # Int64!
    fingerprints: list[Fingerprint] | UnsetType = UNSET  # [Fingerprint!]!

    # Optional fields
    zip_file: BasicFile | None | UnsetType = UNSET  # BasicFile

    # Note: fingerprint field with resolver is not directly supported in Pydantic
    # Users should call fingerprint_resolver(instance, type) directly

    async def to_input(self) -> dict[str, Any]:
        """Convert to GraphQL input.

        Returns:
            Dictionary of input fields for move or set fingerprints operations.
        """
        # Files don't have create/update operations, only move and set fingerprints
        # For move operation - return dict with proper field names
        return {
            "ids": [self.id],
            "destination_folder": None,  # Must be set by caller
            "destination_folder_id": None,  # Must be set by caller
            "destination_basename": self.basename,
        }


class BasicFile(BaseFile):
    """BasicFile type from schema/types/file.graphql.

    Implements BaseFile with no additional fields beyond the base interface."""

    __type_name__ = "BasicFile"


class ImageFile(BaseFile):
    """Image file type from schema/types/file.graphql.

    Implements BaseFile and inherits StashObject through it."""

    __type_name__ = "ImageFile"

    # Required fields
    format: str | UnsetType = UNSET  # String!
    width: int | UnsetType = UNSET  # Int!
    height: int | UnsetType = UNSET  # Int!


class VideoFile(BaseFile):
    """Video file type from schema/types/file.graphql.

    Implements BaseFile and inherits StashObject through it."""

    __type_name__ = "VideoFile"

    # Required fields
    format: str | UnsetType = UNSET  # String!
    width: int | UnsetType = UNSET  # Int!
    height: int | UnsetType = UNSET  # Int!
    duration: float | UnsetType = UNSET  # Float!
    video_codec: str | UnsetType = UNSET  # String!
    audio_codec: str | UnsetType = UNSET  # String!
    frame_rate: float | UnsetType = UNSET  # Float!  # frame_rate in schema
    bit_rate: int | UnsetType = UNSET  # Int!  # bit_rate in schema


# Union type for VisualFile (VideoFile or ImageFile)
VisualFile = VideoFile | ImageFile


class GalleryFile(BaseFile):
    """Gallery file type from schema/types/file.graphql.

    Implements BaseFile with no additional fields and inherits StashObject through it.
    """

    __type_name__ = "GalleryFile"


class ScenePathsType(BaseModel):
    """Scene paths type from schema/types/scene.graphql."""

    screenshot: str | None | UnsetType = UNSET  # String (Resolver)
    preview: str | None | UnsetType = UNSET  # String (Resolver)
    stream: str | None | UnsetType = UNSET  # String (Resolver)
    webp: str | None | UnsetType = UNSET  # String (Resolver)
    vtt: str | None | UnsetType = UNSET  # String (Resolver)
    sprite: str | None | UnsetType = UNSET  # String (Resolver)
    funscript: str | None | UnsetType = UNSET  # String (Resolver)
    interactive_heatmap: str | None | UnsetType = UNSET  # String (Resolver)
    caption: str | None | UnsetType = UNSET  # String (Resolver)


class StashIDInput(StashInput):
    """Input for StashID from schema/types/stash-box.graphql."""

    endpoint: str  # String!
    stash_id: str  # String!
    updated_at: Time | None | UnsetType = UNSET  # Time


class StashID(BaseModel):
    """StashID type from schema/types/stash-box.graphql."""

    endpoint: str | None | UnsetType = UNSET  # String!
    stash_id: str | None | UnsetType = UNSET  # String!
    updated_at: Time | None | UnsetType = UNSET  # Time!


class VideoCaption(BaseModel):
    """Video caption type from schema/types/scene.graphql."""

    language_code: str  # String!
    caption_type: str  # String!


class Folder(StashObject):
    """Folder type from schema/types/file.graphql.

    Note: Inherits from StashObject since it has id, created_at, and updated_at
    fields in the schema, matching the common pattern."""

    __type_name__ = "Folder"

    # Required fields
    path: str | UnsetType = UNSET  # String!
    mod_time: datetime | UnsetType = UNSET  # Time!

    # Optional fields
    parent_folder: Folder | None | UnsetType = UNSET  # Folder
    zip_file: BasicFile | None | UnsetType = UNSET  # BasicFile

    async def to_input(self) -> dict[str, Any]:
        """Convert to GraphQL input.

        Returns:
            Dictionary of input fields for move operation.
        """
        # Folders don't have create/update operations, only move
        # For move operation - return dict with proper field names
        return {
            "ids": [self.id],
            "destination_folder": None,  # Must be set by caller
            "destination_folder_id": None,  # Must be set by caller
            "destination_basename": None,  # Not applicable for folders
        }


class AssignSceneFileInput(StashInput):
    """Input for assigning a file to a scene."""

    scene_id: str  # ID!
    file_id: str  # ID!


class SceneHashInput(StashInput):
    """Input for finding a scene by hash."""

    checksum: str | None | UnsetType = UNSET  # String
    oshash: str | None | UnsetType = UNSET  # String


class FindFilesResultType(StashResult):
    """Result type for finding files from schema/types/file.graphql."""

    count: int  # Int!
    megapixels: float  # Float!
    duration: float  # Float!
    size: int  # Int!
    files: list[BaseFile]  # [BaseFile!]!

    @field_validator("files", mode="before")
    @classmethod
    def _discriminate_file_types(cls, value: Any) -> Any:
        """Discriminate BaseFile interface types based on __typename.

        Converts raw dicts to the appropriate BaseFile subclass based on __typename field.
        """
        if not isinstance(value, list):
            return value

        # Type mapping for BaseFile interface
        type_map: dict[
            str, type[VideoFile] | type[ImageFile] | type[GalleryFile] | type[BasicFile]
        ] = {
            "VideoFile": VideoFile,
            "ImageFile": ImageFile,
            "GalleryFile": GalleryFile,
            "BasicFile": BasicFile,
        }

        result = []
        for item in value:
            if isinstance(item, dict) and "__typename" in item:
                typename = item["__typename"]
                file_type = type_map.get(typename, BaseFile)
                # Construct the correct type from the dict
                result.append(file_type.model_validate(item))
            else:
                # Already constructed or no typename
                result.append(item)

        return result


class FindFoldersResultType(StashResult):
    """Result type for finding folders from schema/types/file.graphql."""

    count: int  # Int!
    folders: list[Folder]  # [Folder!]!



"""Scene types from schema/types/scene.graphql."""

from __future__ import annotations

from datetime import datetime
from enum import Enum
from typing import TYPE_CHECKING

from pydantic import BaseModel, Field

from .base import (
    BulkUpdateIds,
    BulkUpdateStrings,
    RelationshipMetadata,
    StashInput,
    StashObject,
    StashResult,
)
from .files import StashID, StashIDInput, VideoFile
from .scalars import Time
from .unset import UNSET, UnsetType


if TYPE_CHECKING:
    from .gallery import Gallery
    from .group import Group
    from .markers import SceneMarker
    from .performer import Performer
    from .studio import Studio
    from .tag import Tag


class BulkUpdateIdMode(str, Enum):
    """Bulk update ID mode from schema/types/scene.graphql."""

    SET = "SET"
    ADD = "ADD"
    REMOVE = "REMOVE"


class SceneGroup(BaseModel):
    """Scene group type from schema/types/scene.graphql."""

    group: Group | UnsetType = UNSET  # Group!
    scene_index: int | None | UnsetType = UNSET  # Int


class SceneMovie(BaseModel):
    """Scene movie type from schema/types/scene.graphql."""

    movie: Group | UnsetType = UNSET  # Movie! (Movie is deprecated, using Group)
    scene_index: int | None | UnsetType = UNSET  # Int


class VideoCaption(BaseModel):
    """Video caption type from schema/types/scene.graphql."""

    language_code: str | None | UnsetType = UNSET  # String!
    caption_type: str | None | UnsetType = UNSET  # String!


class SceneFileType(BaseModel):
    """Scene file type from schema/types/scene.graphql."""

    size: str | None | UnsetType = UNSET  # String
    duration: float | None | UnsetType = UNSET  # Float
    video_codec: str | None | UnsetType = UNSET  # String
    audio_codec: str | None | UnsetType = UNSET  # String
    width: int | None | UnsetType = UNSET  # Int
    height: int | None | UnsetType = UNSET  # Int
    framerate: float | None | UnsetType = UNSET  # Float
    bitrate: int | None | UnsetType = UNSET  # Int


class ScenePathsType(BaseModel):
    """Scene paths type from schema/types/scene.graphql."""

    screenshot: str | None | UnsetType = UNSET  # String (Resolver)
    preview: str | None | UnsetType = UNSET  # String (Resolver)
    stream: str | None | UnsetType = UNSET  # String (Resolver)
    webp: str | None | UnsetType = UNSET  # String (Resolver)
    vtt: str | None | UnsetType = UNSET  # String (Resolver)
    sprite: str | None | UnsetType = UNSET  # String (Resolver)
    funscript: str | None | UnsetType = UNSET  # String (Resolver)
    interactive_heatmap: str | None | UnsetType = UNSET  # String (Resolver)
    caption: str | None | UnsetType = UNSET  # String (Resolver)


class SceneStreamEndpoint(BaseModel):
    """Scene stream endpoint type from schema/types/scene.graphql."""

    url: str | UnsetType = UNSET  # String!
    mime_type: str | None | UnsetType = UNSET  # String
    label: str | None | UnsetType = UNSET  # String


class SceneGroupInput(StashInput):
    """Input for scene group from schema/types/scene.graphql."""

    group_id: str | UnsetType = UNSET  # ID!
    scene_index: int | None | UnsetType = UNSET  # Int


class SceneUpdateInput(StashInput):
    """Input for updating scenes."""

    # Required fields
    id: str  # ID!

    # Optional fields
    client_mutation_id: str | None | UnsetType = Field(
        default=UNSET, alias="clientMutationId"
    )  # String (camelCase in schema)
    title: str | None | UnsetType = UNSET  # String
    code: str | None | UnsetType = UNSET  # String
    details: str | None | UnsetType = UNSET  # String
    director: str | None | UnsetType = UNSET  # String
    urls: list[str] | None | UnsetType = UNSET  # [String!]
    date: str | None | UnsetType = UNSET  # String
    rating100: int | None | UnsetType = Field(default=UNSET, ge=0, le=100)  # Int
    organized: bool | None | UnsetType = UNSET  # Boolean
    studio_id: str | None | UnsetType = UNSET  # ID (snake_case in schema)
    gallery_ids: list[str] | None | UnsetType = UNSET  # [ID!] (snake_case in schema)
    performer_ids: list[str] | None | UnsetType = UNSET  # [ID!] (snake_case in schema)
    groups: list[SceneGroupInput] | None | UnsetType = UNSET  # [SceneGroupInput!]
    tag_ids: list[str] | None | UnsetType = UNSET  # [ID!] (snake_case in schema)
    cover_image: str | None | UnsetType = (
        UNSET  # String (URL or base64, snake_case in schema)
    )
    stash_ids: list[StashIDInput] | None | UnsetType = (
        UNSET  # [StashIDInput!] (snake_case in schema)
    )
    resume_time: float | None | UnsetType = UNSET  # Float (snake_case in schema)
    play_duration: float | None | UnsetType = UNSET  # Float (snake_case in schema)
    primary_file_id: str | None | UnsetType = UNSET  # ID (snake_case in schema)


class Scene(StashObject):
    """Scene type from schema/types/scene.graphql.

    Note: Inherits from StashObject for implementation convenience, not because
    Scene implements any interface in the schema. StashObject provides common
    functionality like find_by_id, save, and to_input methods."""

    __type_name__ = "Scene"
    __update_input_type__ = SceneUpdateInput
    # No __create_input_type__ - scenes can only be updated, they are created by the server during scanning

    # Fields to track for changes - only fields that can be written via input types
    __tracked_fields__ = {
        "title",  # SceneCreateInput/SceneUpdateInput
        "code",  # SceneCreateInput/SceneUpdateInput
        "details",  # SceneCreateInput/SceneUpdateInput
        "director",  # SceneCreateInput/SceneUpdateInput
        "date",  # SceneCreateInput/SceneUpdateInput
        "studio",  # mapped to studio_id
        "urls",  # SceneCreateInput/SceneUpdateInput
        "organized",  # SceneCreateInput/SceneUpdateInput
        "files",  # mapped to file_ids
        "galleries",  # mapped to gallery_ids
        "groups",  # SceneCreateInput/SceneUpdateInput
        "tags",  # mapped to tag_ids
        "performers",  # mapped to performer_ids
    }

    # Optional fields
    title: str | None | UnsetType = UNSET  # String
    code: str | None | UnsetType = UNSET  # String
    details: str | None | UnsetType = UNSET  # String
    director: str | None | UnsetType = UNSET  # String
    date: str | None | UnsetType = UNSET  # String
    rating100: int | None | UnsetType = Field(
        default=UNSET, ge=0, le=100
    )  # Int (1-100), not used in this client
    o_counter: int | None | UnsetType = Field(
        default=UNSET, ge=0
    )  # Int, not used in this client
    studio: Studio | None | UnsetType = UNSET  # Studio
    interactive: bool | None | UnsetType = UNSET  # Boolean
    interactive_speed: int | None | UnsetType = Field(
        default=UNSET, alias="interactiveSpeed"
    )  # Int
    # created_at and updated_at inherited from StashObject
    last_played_at: Time | None | UnsetType = Field(
        default=UNSET, alias="lastPlayedAt"
    )  # Time
    resume_time: float | None | UnsetType = Field(
        default=UNSET, alias="resumeTime"
    )  # Float
    play_duration: float | None | UnsetType = Field(
        default=UNSET, alias="playDuration"
    )  # Float
    play_count: int | None | UnsetType = Field(
        default=UNSET, ge=0, alias="playCount"
    )  # Int
    play_history: list[Time] | None | UnsetType = Field(
        default=UNSET, alias="playHistory"
    )  # [Time!]
    o_history: list[Time] | None | UnsetType = Field(
        default=UNSET, alias="oHistory"
    )  # [Time!]

    # Required fields
    urls: list[str] | UnsetType = UNSET  # [String!]!
    organized: bool | UnsetType = UNSET  # Boolean!
    files: list[VideoFile] | UnsetType = UNSET  # [VideoFile!]!
    paths: ScenePathsType | UnsetType = UNSET  # ScenePathsType! (Resolver)
    scene_markers: list[SceneMarker] | UnsetType = UNSET  # [SceneMarker!]!
    galleries: list[Gallery] | UnsetType = UNSET  # [Gallery!]!
    groups: list[SceneGroup] | UnsetType = UNSET  # [SceneGroup!]!
    tags: list[Tag] | UnsetType = UNSET  # [Tag!]!
    performers: list[Performer] | UnsetType = UNSET  # [Performer!]!
    stash_ids: list[StashID] | UnsetType = UNSET  # [StashID!]!
    scene_streams: list[SceneStreamEndpoint] | UnsetType = Field(
        default=UNSET, alias="sceneStreams"
    )  # [SceneStreamEndpoint!]! (Return valid stream paths)

    # Optional lists
    captions: list[VideoCaption] | None | UnsetType = (
        UNSET  # [VideoCaption!] - nullable list
    )

    # Relationship definitions with their mappings
    __relationships__ = {
        # Pattern A: Direct field relationships (many-to-many)
        "galleries": RelationshipMetadata(
            target_field="gallery_ids",
            is_list=True,
            query_field="galleries",
            inverse_type="Gallery",
            inverse_query_field="scenes",
            query_strategy="direct_field",
            notes="Backend auto-syncs both scene.galleries and gallery.scenes",
        ),
        "performers": RelationshipMetadata(
            target_field="performer_ids",
            is_list=True,
            query_field="performers",
            inverse_type="Performer",
            inverse_query_field="scenes",
            query_strategy="direct_field",
            notes="Backend auto-syncs scene.performers and performer.scenes",
        ),
        "tags": RelationshipMetadata(
            target_field="tag_ids",
            is_list=True,
            query_field="tags",
            inverse_type="Tag",
            inverse_query_field=None,  # Tag only has scene_count, not scenes list
            query_strategy="direct_field",
            notes="Tag has scene_count resolver, not direct scenes list",
        ),
        # Pattern B: Filter query relationship (many-to-one)
        "studio": RelationshipMetadata(
            target_field="studio_id",
            is_list=False,
            query_field="studio",
            inverse_type="Studio",
            inverse_query_field=None,  # No direct scenes field on Studio
            query_strategy="filter_query",
            filter_query_hint="findScenes(scene_filter={studios: {value: [studio_id]}})",
            notes="Studio uses filter-based queries. Use client.find_scenes(scene_filter=...) for inverse.",
        ),
        # Special case: Complex transform for StashID
        "stash_ids": RelationshipMetadata(
            target_field="stash_ids",
            is_list=True,
            transform=lambda s: StashIDInput(endpoint=s.endpoint, stash_id=s.stash_id),
            query_field="stash_ids",
            notes="Requires transform to StashIDInput for mutations",
        ),
    }

    # Field definitions with their conversion functions
    __field_conversions__ = {
        "title": str,
        "code": str,
        "details": str,
        "director": str,
        "urls": list,
        "rating100": int,
        "organized": bool,
        "date": lambda d: (
            d.strftime("%Y-%m-%d")
            if isinstance(d, datetime)
            else (
                datetime.fromisoformat(d).strftime("%Y-%m-%d")
                if isinstance(d, str)
                else None
            )
        ),
    }

    # =========================================================================
    # Convenience Helper Methods for Bidirectional Relationships
    # =========================================================================

    async def add_to_gallery(self, gallery: Gallery) -> None:
        """Add scene to gallery (syncs inverse automatically, call save() to persist)."""
        await self._add_to_relationship("galleries", gallery)

    async def remove_from_gallery(self, gallery: Gallery) -> None:
        """Remove scene from gallery (syncs inverse automatically, call save() to persist)."""
        await self._remove_from_relationship("galleries", gallery)

    async def add_performer(self, performer: Performer) -> None:
        """Add performer to scene (syncs inverse automatically, call save() to persist)."""
        await self._add_to_relationship("performers", performer)

    async def remove_performer(self, performer: Performer) -> None:
        """Remove performer from scene (syncs inverse automatically, call save() to persist)."""
        await self._remove_from_relationship("performers", performer)

    async def add_tag(self, tag: Tag) -> None:
        """Add tag to scene (syncs inverse automatically, call save() to persist)."""
        await self._add_to_relationship("tags", tag)

    async def remove_tag(self, tag: Tag) -> None:
        """Remove tag from scene (syncs inverse automatically, call save() to persist)."""
        await self._remove_from_relationship("tags", tag)

    def set_studio(self, studio: Studio | None) -> None:
        """Set scene studio (call save() to persist)."""
        self.studio = studio


class SceneMovieID(BaseModel):
    """Movie ID with scene index."""

    movie_id: str | UnsetType = UNSET  # ID!
    scene_index: str | None | UnsetType = UNSET  # String


class SceneParserInput(StashInput):
    """Input for scene parser from schema/types/scene.graphql."""

    ignore_words: list[str] | None | UnsetType = Field(
        default=UNSET, alias="ignoreWords"
    )  # [String!]
    whitespace_characters: str | None | UnsetType = Field(
        default=UNSET, alias="whitespaceCharacters"
    )  # String
    capitalize_title: bool | None | UnsetType = Field(
        default=UNSET, alias="capitalizeTitle"
    )  # Boolean
    ignore_organized: bool | None | UnsetType = Field(
        default=UNSET, alias="ignoreOrganized"
    )  # Boolean


class FindScenesResultType(StashResult):
    """Result type for finding scenes from schema/types/scene.graphql.

    Fields:
    count: Total number of scenes
    duration: Total duration in seconds
    filesize: Total file size in bytes
    scenes: List of scenes
    """

    count: int | UnsetType = UNSET  # Int!
    duration: float | UnsetType = UNSET  # Float!
    filesize: float | UnsetType = UNSET  # Float!
    scenes: list[Scene] | UnsetType = UNSET  # [Scene!]!


class SceneParserResult(BaseModel):
    """Result type for scene parser from schema/types/scene.graphql."""

    scene: Scene | UnsetType = UNSET  # Scene!
    title: str | None | UnsetType = UNSET  # String
    code: str | None | UnsetType = UNSET  # String
    details: str | None | UnsetType = UNSET  # String
    director: str | None | UnsetType = UNSET  # String
    url: str | None | UnsetType = UNSET  # String
    date: str | None | UnsetType = UNSET  # String
    rating100: int | None | UnsetType = Field(
        default=UNSET, ge=0, le=100
    )  # Int (1-100)
    studio_id: str | None | UnsetType = UNSET  # ID
    gallery_ids: list[str] | None | UnsetType = UNSET  # [ID!]
    performer_ids: list[str] | None | UnsetType = UNSET  # [ID!]
    movies: list[SceneMovieID] | None | UnsetType = UNSET  # [SceneMovieID!]
    tag_ids: list[str] | None | UnsetType = UNSET  # [ID!]


class SceneCreateInput(StashInput):
    """Input for creating scenes."""

    # All fields optional
    title: str | None | UnsetType = UNSET  # String
    code: str | None | UnsetType = UNSET  # String
    details: str | None | UnsetType = UNSET  # String
    director: str | None | UnsetType = UNSET  # String
    urls: list[str] | None | UnsetType = UNSET  # [String!]
    date: str | None | UnsetType = UNSET  # String
    rating100: int | None | UnsetType = Field(default=UNSET, ge=0, le=100)  # Int
    organized: bool | None | UnsetType = UNSET  # Boolean
    studio_id: str | None | UnsetType = UNSET  # ID
    gallery_ids: list[str] | None | UnsetType = UNSET  # [ID!]
    performer_ids: list[str] | None | UnsetType = UNSET  # [ID!]
    groups: list[SceneGroupInput] | None | UnsetType = UNSET  # [SceneGroupInput!]
    tag_ids: list[str] | None | UnsetType = UNSET  # [ID!]
    cover_image: str | None | UnsetType = UNSET  # String (URL or base64)
    stash_ids: list[StashIDInput] | None | UnsetType = UNSET  # [StashIDInput!]
    file_ids: list[str] | None | UnsetType = UNSET  # [ID!]


class BulkSceneUpdateInput(StashInput):
    """Input for bulk updating scenes."""

    # Optional fields
    client_mutation_id: str | None | UnsetType = Field(
        default=UNSET, alias="clientMutationId"
    )  # String (camelCase in schema)
    ids: list[str] | UnsetType = UNSET  # [ID!]
    title: str | None | UnsetType = UNSET  # String
    code: str | None | UnsetType = UNSET  # String
    details: str | None | UnsetType = UNSET  # String
    director: str | None | UnsetType = UNSET  # String
    urls: BulkUpdateStrings | None | UnsetType = UNSET  # BulkUpdateStrings
    date: str | None | UnsetType = UNSET  # String
    rating100: int | None | UnsetType = Field(
        default=UNSET, ge=0, le=100
    )  # Int (1-100)
    organized: bool | None | UnsetType = UNSET  # Boolean
    studio_id: str | None | UnsetType = UNSET  # ID (snake_case in schema)
    gallery_ids: BulkUpdateIds | None | UnsetType = (
        UNSET  # BulkUpdateIds (snake_case in schema)
    )
    performer_ids: BulkUpdateIds | None | UnsetType = (
        UNSET  # BulkUpdateIds (snake_case in schema)
    )
    tag_ids: BulkUpdateIds | None | UnsetType = (
        UNSET  # BulkUpdateIds (snake_case in schema)
    )
    group_ids: BulkUpdateIds | None | UnsetType = (
        UNSET  # BulkUpdateIds (snake_case in schema)
    )


class SceneParserResultType(BaseModel):
    """Result type for scene parser from schema/types/scene.graphql."""

    count: int | UnsetType = UNSET  # Int!
    results: list[SceneParserResult] | UnsetType = UNSET  # [SceneParserResult!]!


class AssignSceneFileInput(StashInput):
    """Input for assigning a file to a scene from schema/types/scene.graphql."""

    scene_id: str | UnsetType = UNSET  # ID!
    file_id: str | UnsetType = UNSET  # ID!


class SceneDestroyInput(StashInput):
    """Input for destroying a scene from schema/types/scene.graphql."""

    id: str  # ID!
    delete_file: bool | None | UnsetType = UNSET  # Boolean
    delete_generated: bool | None | UnsetType = UNSET  # Boolean


class SceneHashInput(StashInput):
    """Input for scene hash from schema/types/scene.graphql."""

    checksum: str | None | UnsetType = UNSET  # String
    oshash: str | None | UnsetType = UNSET  # String


class SceneMergeInput(StashInput):
    """Input for merging scenes from schema/types/scene.graphql."""

    source: list[str] | UnsetType = UNSET  # [ID!]!
    destination: str | UnsetType = UNSET  # ID!
    values: SceneUpdateInput | None | UnsetType = UNSET  # SceneUpdateInput
    play_history: bool | None | UnsetType = UNSET  # Boolean
    o_history: bool | None | UnsetType = UNSET  # Boolean


class SceneMovieInput(StashInput):
    """Input for scene movie from schema/types/scene.graphql."""

    movie_id: str | UnsetType = UNSET  # ID!
    scene_index: int | None | UnsetType = UNSET  # Int


class ScenesDestroyInput(StashInput):
    """Input for destroying multiple scenes from schema/types/scene.graphql."""

    ids: list[str] | UnsetType = UNSET  # [ID!]!
    delete_file: bool | None | UnsetType = UNSET  # Boolean
    delete_generated: bool | None | UnsetType = UNSET  # Boolean


class HistoryMutationResult(BaseModel):
    """Result type for history mutation from schema/types/scene.graphql."""

    count: int | UnsetType = UNSET  # Int!
    history: list[Time] | UnsetType = UNSET  # [Time!]!
